/* sysBC635.c - Datum BC635/7 PCI card initialization */

/*
  *******************************************************************************
** Copyright (C) 1996 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/


/*
modification history
--------------------
*/

/*
Description

This file contains BSI Specific routines.

The constant PCI2DRAM_BASE_ADRS must be defined in the BSP.
*/

#include "vxWorks.h"
#include "logLib.h"
#include "sysLib.h"
#include "config.h"
#include "arch/ppc/archPpc.h"
#include "mv2600.h"
#include "drv/pci/pciConfigLib.h"
#include "bc635P.h"


/* BAR0 address - PCI bus address for base of bc635/7 registers */
#define TFP_PCI_REGS_BASE_ADRS  (PCI_MEM_ADRS + PMC_DEV_SPACE)

/* BAR1 address - PCI bus address for base of bc635/7 dpram space */
#define TFP_PCI_DPRAM_BASE_ADRS (TFP_PCI_REGS_BASE_ADRS + \
                                 TFP_DPRAM_ADRS_START_OFFSET)


/* global declarations */

u_int   bc635PciMemOffset;


/* local declarations */

STATUS 
bc635PciDevConfig (int pciBusNo, int pciDevNo, int pciFuncNo,
                   ULONG bc635PciDpramBaseAdrs,
                   ULONG bc635PciRegsBaseAdrs,
                   ULONG command);

/* global declarations */

int     bc635Debug = 0;
int     bc635IntsDebug = 0;

/*******************************************************************************
*
* sysBC635Init - initialize BC635 PCI device support
*
* This routine ....
* next line.
*
* RETURNS: OK, or ERROR if
* next line.
*
*/

STATUS 
sysBC635Init ()
{
    int     pciBusNo;
    int     pciDevNo;
    int     pciFuncNo;

    /* verify device is present */
    if (pciFindDevice (BC635_PCI_VENDOR_ID, BC635_PCI_DEVICE_ID, 0,
                       &pciBusNo, &pciDevNo, &pciFuncNo) != ERROR)
    {
        TFP_DEBUG_MSG ("Found BC635/7\n", 0, 0, 0, 0, 0, 0);
    }
    else
    {
        TFP_MSG ("BC635/7 not found\n", 0, 0, 0, 0, 0, 0);
        return (ERROR);
    }

    /*
     * Perform PCI device initialization by: - programming BAR0 and BAR1 into
     * card. - enabling mem access to card.
     */
    if (bc635PciDevConfig (pciBusNo, pciDevNo, pciFuncNo,
                           TFP_PCI_REGS_BASE_ADRS,
                           TFP_PCI_DPRAM_BASE_ADRS,
                           (PCI_CMD_MASTER_ENABLE |
                            PCI_CMD_MEM_ENABLE |
                            PCI_CMD_IO_ENABLE)) == ERROR)
    {
        TFP_MSG ("bc635PciDevConfig failed\n", 0, 0, 0, 0, 0, 0);
        return (ERROR);
    }

    /* driver must add this to BAR0 & BAR1 to access card via CPU */
    bc635PciMemOffset = PCI2DRAM_BASE_ADRS;

    return (OK);

}


/******************************************************************************
*
* bc635PciDevConfig - configure a device on a PCI bus
*
* This routine configures a device that is on a Peripheral Component
* Interconnect (PCI) bus by writing to the configuration header of the
* selected device.
* It first disables the device by clearing the command register in the
* configuration header.  It then sets the I/O and/or memory space base
* address registers, the latency timer value and the cache line size.
* Finally, it re-enables the device by loading the command register with
* the specified command.
*
* NOTE: This routine is designed for Type 0 PCI Configuration Headers ONLY.
*       It is NOT usable for configuring, for example, a PCI-to-PCI bridge.
*
* RETURNS: OK always.
*/

STATUS  bc635PciDevConfig
        (
                 int pciBusNo,  /* PCI bus number */
                 int pciDevNo,  /* PCI device number */
                 int pciFuncNo, /* PCI function number */
                 ULONG bc635PciRegsBaseAdrs,    /* device regs base adrs */
                 ULONG bc635PciDpramBaseAdrs,   /* device dpram base adrs */
                 ULONG command  /* command to issue */
)
{
    UINT32  tmp32;

    /*
     * Disable device by clearing its command register field in its
     * configuration header.
     */

    pciConfigOutWord (pciBusNo, pciDevNo, pciFuncNo, PCI_CFG_COMMAND, 0);


    /* Set BAR0 (bc635 regs base adrs) */

    /* write all f's and read back value */
    pciConfigOutLong (pciBusNo, pciDevNo, pciFuncNo,
                      PCI_CFG_BASE_ADDRESS_0, 0xffffffff);

    pciConfigInLong (pciBusNo, pciDevNo, pciFuncNo,
                     PCI_CFG_BASE_ADDRESS_0, &tmp32);

    TFP_DEBUG_MSG ("Set REG base(pci) = 0x%08X\n",
                   bc635PciRegsBaseAdrs, 0, 0, 0, 0, 0);

    pciConfigOutLong (pciBusNo, pciDevNo, pciFuncNo,
                      PCI_CFG_BASE_ADDRESS_0, bc635PciRegsBaseAdrs);


    /* Set BAR1 (bc635 dpram space base adrs) */

    /* write all f's and read back value */
    pciConfigOutLong (pciBusNo, pciDevNo, pciFuncNo,
                      PCI_CFG_BASE_ADDRESS_1, 0xffffffff);

    pciConfigInLong (pciBusNo, pciDevNo, pciFuncNo,
                     PCI_CFG_BASE_ADDRESS_1, &tmp32);

    TFP_DEBUG_MSG ("Set DPRAM base(pci) = 0x%08X\n",
                   bc635PciDpramBaseAdrs, 0, 0, 0, 0, 0);

    pciConfigOutLong (pciBusNo, pciDevNo, pciFuncNo,
                      PCI_CFG_BASE_ADDRESS_1, bc635PciDpramBaseAdrs);



    /* Enable the device's capabilities as specified */

    pciConfigOutWord (pciBusNo, pciDevNo, pciFuncNo, PCI_CFG_COMMAND,
                      (UINT16) command);

    return (OK);
}
